"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.formatTableOfContents = void 0;
/**
 * 목차 구조를 계층적으로 변환하는 함수
 */
const formatTableOfContents = (tableOfContents, reporter) => {
    if (!tableOfContents || tableOfContents.length === 0) {
        return [];
    }
    try {
        // Deep clone to avoid modifying original
        const tocClone = JSON.parse(JSON.stringify(tableOfContents));
        // Ensure TOC entries are sorted by level first, then by appearance order
        const sortedToc = tocClone.sort((a, b) => {
            // Level 기준 정렬
            if ((a.level || 0) !== (b.level || 0)) {
                return (a.level || 0) - (b.level || 0);
            }
            // Original order is preserved
            return 0;
        });
        // Add contextTitle where missing
        for (const entry of sortedToc) {
            if (!entry.contextTitle) {
                entry.contextTitle = entry.title;
            }
        }
        // Build the heading hierarchy based on level and parentHash
        const headingsByHash = new Map();
        const childrenByParent = new Map();
        const validHeadingsByLevel = new Map();
        // Initialize valid headings collections for each level
        for (let level = 1; level <= 3; level++) {
            validHeadingsByLevel.set(level, new Set());
        }
        // First pass: Collect all headings by hash
        for (const entry of sortedToc) {
            headingsByHash.set(entry.hash, entry);
            // Store valid headings by level
            if (entry.level && entry.level >= 1 && entry.level <= 3) {
                validHeadingsByLevel.get(entry.level).add(entry.hash);
            }
        }
        // Second pass: Validate parent-child relationships
        const invalidHashesCollectionByReason = new Map();
        const addInvalidHash = (hash, reason) => {
            if (!invalidHashesCollectionByReason.has(reason)) {
                invalidHashesCollectionByReason.set(reason, new Set());
            }
            invalidHashesCollectionByReason.get(reason).add(hash);
        };
        // Build parent-child relationship map
        for (const entry of sortedToc) {
            if (entry.parentHash) {
                // Check if parent exists
                const parentExists = headingsByHash.has(entry.parentHash);
                if (parentExists) {
                    // Add this entry as a child of its parent
                    if (!childrenByParent.has(entry.parentHash)) {
                        childrenByParent.set(entry.parentHash, new Set());
                    }
                    childrenByParent.get(entry.parentHash).add(entry.hash);
                }
                else {
                    // Invalid parent reference
                    addInvalidHash(entry.hash, "orphaned");
                }
                // Validate level relationships
                const parent = headingsByHash.get(entry.parentHash);
                if (parent && entry.level && parent.level) {
                    if (entry.level <= parent.level) {
                        // Invalid level hierarchy (e.g. H2 with H2 or H3 parent)
                        addInvalidHash(entry.hash, "invalid_level");
                    }
                    if (entry.level > parent.level + 1) {
                        // Skip levels (e.g. H1 -> H3 directly)
                        // This is actually valid in the new implementation, but we want to note it
                        reporter.info(`Found H${entry.level} heading "${entry.title}" directly under H${parent.level}`);
                    }
                }
            }
            else if (entry.level && entry.level > 1) {
                // No parent hash but not a top-level heading
                addInvalidHash(entry.hash, "missing_parent");
            }
        }
        // Identify common command names for special handling
        const commonCommandPattern = /^(cp|mv|rm|find|locate|grep|curl|wget|ls|chmod|chown|ps|sed|awk|cat|service|file|stat)$/i;
        // Third pass: Identify structurally invalid entries based on context
        const invalidEntries = new Set();
        // Process invalid entries from previous validations
        for (const [reason, hashes] of invalidHashesCollectionByReason.entries()) {
            if (reason === "orphaned" ||
                reason === "invalid_level" ||
                reason === "missing_parent") {
                for (const hash of hashes) {
                    invalidEntries.add(hash);
                }
            }
        }
        // Find server/service sections
        const serverSections = sortedToc.filter((entry) => entry.level === 1 &&
            (entry.title.includes("서버") ||
                entry.title.includes("Server") ||
                entry.title.includes("프록시")));
        // Process special handling for server sections
        for (const serverSection of serverSections) {
            // Get direct children of this server section
            const directChildren = childrenByParent.get(serverSection.hash) || new Set();
            // Check each child for common command pattern matching
            for (const childHash of directChildren) {
                const child = headingsByHash.get(childHash);
                if (child && commonCommandPattern.test(child.title)) {
                    // Check if this child has valid descendants or content
                    const childHasChildren = childrenByParent.has(childHash) &&
                        childrenByParent.get(childHash).size > 0;
                    // Special check for service section headings - common commands shouldn't be here
                    // if they don't have meaningful content
                    if (!childHasChildren) {
                        invalidEntries.add(childHash);
                        // Also, mark children as invalid if there are any
                        if (childrenByParent.has(childHash)) {
                            for (const grandChildHash of childrenByParent.get(childHash)) {
                                invalidEntries.add(grandChildHash);
                            }
                        }
                    }
                }
            }
        }
        // Ensure the correct context titles reflect the parent relationship
        for (const entry of sortedToc) {
            if (entry.level === 2 && entry.parentHash) {
                const parent = headingsByHash.get(entry.parentHash);
                if (parent) {
                    entry.contextTitle = `${entry.title} (${parent.title})`;
                }
            }
            else if (entry.level === 3 && entry.parentHash) {
                // For H3, prefer its direct parent, whether H2 or H1
                const parent = headingsByHash.get(entry.parentHash);
                if (parent) {
                    entry.contextTitle = `${entry.title} (${parent.title})`;
                }
            }
        }
        // Create a filtered TOC without invalid entries
        const validToc = sortedToc.filter((entry) => !invalidEntries.has(entry.hash));
        if (invalidEntries.size > 0) {
            reporter.info(`Removed ${invalidEntries.size} invalid TOC entries`);
        }
        return validToc;
    }
    catch (error) {
        reporter.error(`Error formatting table of contents: ${error}`);
        return tableOfContents; // Return original on error
    }
};
exports.formatTableOfContents = formatTableOfContents;
