"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.processTableOfContents = exports.resetTocContext = void 0;
const crypto_1 = require("crypto");
// Hash 생성 시 사용할 캐시
const hashCache = new Map();
// 계층별 컨텍스트 저장
const headingStack = [];
let lastHeadingByLevel = [null, null, null, null]; // 인덱스 1부터 3까지 사용
const resetTocContext = () => {
    headingStack.length = 0;
    lastHeadingByLevel = [null, null, null, null];
    hashCache.clear();
};
exports.resetTocContext = resetTocContext;
// 필요한 수준의 상위 헤딩 찾기
const findParentHeading = (currentLevel) => {
    // 자신보다 낮은 레벨의 가장 가까운 헤딩 찾기 (1 < 2 < 3)
    for (let level = currentLevel - 1; level > 0; level--) {
        if (lastHeadingByLevel[level]) {
            return lastHeadingByLevel[level];
        }
    }
    return null;
};
const generateHash = (text) => {
    return `link-${text
        .replace(/[^a-zA-Z0-9가-힣\s-_]/g, "")
        .trim()
        .replace(/\s+/g, "-")
        .toLowerCase()}-${(0, crypto_1.randomUUID)().substring(0, 4)}`;
};
const processTableOfContents = async (block, tableOfContents) => {
    if (["heading_1", "heading_2", "heading_3"].includes(block.type) &&
        block[block.type]?.rich_text?.length > 0) {
        const plainText = block[block.type]?.rich_text?.[0]?.plain_text || "";
        // 헤딩 레벨 추출 (heading_1 -> 1, heading_2 -> 2, heading_3 -> 3)
        const level = parseInt(block.type.split("_")[1]);
        // 현재 헤딩의 컨텍스트 ID 생성
        const contextualId = `h${level}-${plainText}`;
        // 해시 생성 또는 캐시에서 가져오기
        let hash;
        if (hashCache.has(contextualId)) {
            hash = hashCache.get(contextualId);
        }
        else {
            hash = generateHash(plainText);
            hashCache.set(contextualId, hash);
        }
        block.hash = hash;
        // 계층 구조 관리 - 현재 레벨보다 높은 모든 하위 헤딩 컨텍스트 삭제
        // 예: H1이 나오면 H2, H3 컨텍스트 초기화
        for (let i = level + 1; i < lastHeadingByLevel.length; i++) {
            lastHeadingByLevel[i] = null;
        }
        // 부모 헤딩 찾기
        const parentHeading = findParentHeading(level);
        const parentHash = parentHeading ? parentHeading.hash : "";
        // 부모 헤딩 스택에 자식으로 등록
        if (parentHeading) {
            parentHeading.children.push(hash);
        }
        // 컨텍스트 제목 생성
        let contextTitle = plainText;
        // 부모 컨텍스트 추가
        if (level > 1 && parentHeading) {
            contextTitle = `${plainText} (${parentHeading.title})`;
        }
        // 자신을 현재 레벨의 마지막 헤딩으로 등록
        const currentHeading = {
            hash,
            title: plainText,
            level,
            children: [],
        };
        lastHeadingByLevel[level] = currentHeading;
        // 중복 체크 - 동일한 hash가 이미 존재하는지 확인
        const existingTocIndex = tableOfContents.findIndex((toc) => toc.hash === hash);
        if (existingTocIndex === -1) {
            // TOC 항목 추가
            tableOfContents.push({
                type: block.type,
                hash,
                title: plainText,
                contextTitle,
                parentHash,
                level,
            });
        }
    }
};
exports.processTableOfContents = processTableOfContents;
